@@ -52,8 +52,8 @@ $vex_classes = scalar(@vex_class);
 for ($c = 0; $c < $vex_classes; $c++) {
     $vexmap{$vex_class[$c]} = $c;
     for ($m = 0; $m < 32; $m++) {
-	for ($lp = 0; $lp < 8; $lp++) {
-	    push(@vexlist, sprintf("%s%02X%01X", $vex_class[$c], $m, $lp));
+	for ($p = 0; $p < 4; $p++) {
+	    push(@vexlist, sprintf("%s%02X%01X", $vex_class[$c], $m, $p));
 	}
     }
 }
@@ -308,33 +308,30 @@ if ( !defined($output) || $output eq 'd' ) {
 	    if ($is_prefix{$nn}) {
 		die "$fname: ambiguous decoding of $nn\n"
 		    if (defined($dinstables{$nn}));
-		printf D "    { itable_%s, -1 },\n", $nn;
+		printf D "    /* 0x%02x */ { itable_%s, -1 },\n", $c, $nn;
 	    } elsif (defined($dinstables{$nn})) {
-		printf D "    { itable_%s, %u },\n",
-		$nn, scalar(@{$dinstables{$nn}});
+		printf D "    /* 0x%02x */ { itable_%s, %u },\n", $c,
+		    $nn, scalar(@{$dinstables{$nn}});
 	    } else {
-		printf D "    { NULL, 0 },\n";
+		printf D "    /* 0x%02x */ { NULL, 0 },\n", $c;
 	    }
 	}
 	print D "};\n";
     }
 
-    printf D "\nconst struct disasm_index * const itable_vex[%d][32][8] =\n",
+    printf D "\nconst struct disasm_index * const itable_vex[%d][32][4] =\n",
         $vex_classes;
     print D "{\n";
     for ($c = 0; $c < $vex_classes; $c++) {
 	print D "    {\n";
 	for ($m = 0; $m < 32; $m++) {
-	    print D "        {\n";
-	    for ($lp = 0; $lp < 8; $lp++) {
-		$vp = sprintf("%s%02X%01X", $vex_class[$c], $m, $lp);
-		if ($is_prefix{$vp}) {
-		    printf D "            itable_%s,\n", $vp;
-		} else {
-		    print  D "            NULL,\n";
-		}
+	    print D "        { ";
+	    for ($p = 0; $p < 4; $p++) {
+		$vp = sprintf("%s%02X%01X", $vex_class[$c], $m, $p);
+		printf D "%-15s",
+		    ($is_prefix{$vp} ? sprintf("itable_%s,", $vp) : 'NULL,');
 	    }
-	    print D "        },\n";
+	    print D "},\n";
 	}
 	print D "    },\n";
     }
@@ -628,7 +625,7 @@ sub startseq($$) {
 	    $wlp = shift(@codes);
 	    $c = ($m >> 6);
 	    $m = $m & 31;
-	    $prefix .= sprintf('%s%02X%01X', $vex_class[$c], $m, $wlp & 7);
+	    $prefix .= sprintf('%s%02X%01X', $vex_class[$c], $m, $wlp & 3);
 	} elsif ($c0 >= 0172 && $c0 <= 174) {
 	    shift(@codes);	# Skip is4 control byte
 	} else {
@@ -761,15 +758,17 @@ sub byte_code_compile($$) {
 	    my @subops = split(/\./, $op);
 	    shift @subops;	# Drop prefix
 	    foreach $oq (@subops) {
-		if ($oq eq '128' || $oq eq 'l0') {
+		if ($oq eq '128' || $oq eq 'l0' || $oq eq 'lz') {
 		    $l = 0;
 		} elsif ($oq eq '256' || $oq eq 'l1') {
 		    $l = 1;
+		} elsif ($oq eq 'lig') {
+		    $l = 2;
 		} elsif ($oq eq 'w0') {
 		    $w = 0;
 		} elsif ($oq eq 'w1') {
 		    $w = 1;
-		} elsif ($oq eq 'wx') {
+		} elsif ($oq eq 'wig') {
 		    $w = 2;
 		} elsif ($oq eq 'ww') {
 		    $w = 3;
@@ -805,7 +804,7 @@ sub byte_code_compile($$) {
 		die "$fname: $line: 'v' operand without vex.nds or vex.ndd\n";
 	    }
 	    push(@codes, defined($oppos{'v'}) ? 0260+($oppos{'v'} & 3) : 0270,
-		 ($c << 6)+$m, ($w << 3)+($l << 2)+$p);
+		 ($c << 6)+$m, ($w << 4)+($l << 2)+$p);
 	    $prefix_ok = 0;
 	} elsif ($op =~ /^\/drex([01])$/) {
 	    my $oc0 = $1;
