@@ -45,8 +45,9 @@
 #include "nasmlib.h"
 #include "preproc.h"
 #include "insnsi.h"		/* For enum opcode */
-#include "directives.h"		/* For enum directive */
+#include "directiv.h"		/* For enum directive */
 #include "opflags.h"
+#include "regs.h"
 
 #define NO_SEG -1L              /* null segment value */
 #define SEG_ABS 0x40000000L     /* mask for far-absolute segments */
@@ -101,6 +102,7 @@ enum out_type {
     OUT_RAWDATA,		/* Plain bytes */
     OUT_ADDRESS,		/* An address (symbol value) */
     OUT_RESERVE,		/* Reserved bytes (RESB et al) */
+    OUT_REL1ADR,		/* 1-byte relative address */
     OUT_REL2ADR,		/* 2-byte relative address */
     OUT_REL4ADR,		/* 4-byte relative address */
     OUT_REL8ADR,		/* 8-byte relative address */
@@ -417,8 +419,11 @@ enum {
  * -----------------------------------------------------------
  */
 
-/* Register names automatically generated from regs.dat */
-#include "regs.h"
+/* Verify value to be a valid register */
+static inline bool is_register(int reg)
+{
+    return reg >= EXPR_REG_START && reg < REG_ENUM_LIMIT;
+}
 
 enum ccode {			/* condition code names */
     C_A, C_AE, C_B, C_BE, C_C, C_E, C_G, C_GE, C_L, C_LE, C_NA, C_NAE,
@@ -681,6 +686,12 @@ struct ofmt {
      */
     int32_t (*section) (char *name, int pass, int *bits);
 
+    /*
+     * This procedure is called to modify section alignment,
+     * note there is a trick, the alignment can only increase
+     */
+    void (*sectalign)(int32_t seg, unsigned int value);
+
     /*
      * This procedure is called to modify the segment base values
      * returned from the SEG operator. It is given a segment base
